<?php

/**
 * Base controller that all other controllers inherit from.
 *
 * Contains helper methods
 */
abstract class base_controller {
	/** The name of the current page, if other than the controller name */
	public $name;
	/** input_nacoma_Model */
	protected $input;

	/** Base constructor - we probably want to do something here at some point... */
	function __construct($input) {}

	/** Index action */
	abstract function index();

	/**
	 * This will store message in session until it is read
	 *
	 * @param $message string
	 * @throws InvalidArgumentException if session is not started
	 */
	protected function flash($message) {
		if(!session_id()) {
			throw new InvalidArgumentException("Sessions not started, could not flash '$message'");
		}
		if(!isset($_SESSION['nacoma_flash'])) {
			$_SESSION['nacoma_flash'] = array();
		}
		$_SESSION['nacoma_flash'][] = $message;
	}

	/**
	 * Do a HTTP Redirect to the specified controller and/or action
	 *
	 * @param $controller string|base_controller
	 * @param $action string = 'index'
	 * @throws InvalidArgumentException
	 */
	static function redirect($controller, $action = 'index') {
		if($controller instanceof base_controller) {
			$controller = preg_replace("~_controller$~", null, get_class($controller));
		}
		if(!is_string($controller)) {
			throw new InvalidArgumentException('$controller must be string or instance of base_controller');
		}
		$path = "edit_special.php/".$controller;
		if($action != 'index') {
			$path .= "?action=$action";
		}
		header("Location: $path");
		exit(0);
	}
}
