<?php

require_once 'base_controller.php';
require_once('op5/config.php');

/**
 * Handles groups of users
 */
class group_controller extends base_controller {
	public $name = "Grouprights";

	/**
	 * Verify user permissions
	 */
	function __construct($input) {
		$access = get_nacoma_access('access_rights');
		if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
			deny_access("You need full 'access_rights' permissions to modify permissions.");
		}
		$this->input = $input;
	}

	function index() {
		global $page;
		$groups = usergroup_nacoma_Model::get_groups();
		$group_available = Op5Auth::instance()->groups_available( array_keys( $groups ) );
		$access_rights = Op5Authorization::get_all_auth_levels();
		require 'templates/group.php';
	}

	/**
	 * Renames and/or deletes user group memberships
	 *
	 * @return void
	 **/
	function modify_user_memberships($users, $old_name, $new_name = false)
	{
		foreach ($users as $idx => $user) {
			if (isset($user['groups'])) {
				$groups = array();
				foreach ($user['groups'] as $group) {
					if ($new_name !== false && $group === $old_name) {
						// New name was given, modify the group name
						$groups[] = $new_name;
					} else if ($group !== $old_name) {
						$groups[] = $group;
					}
					// Else the group will be removed
				}
				$users[$idx]['groups'] = $groups;
			}
		}
		return $users;
	}

	/** Modifies groups on your behalf */
	function submit() {
		$saved = array();
		$removed = array();
		// Fetch users to match group memberships
		$users = Op5Config::instance()->getConfig('auth_users');
		foreach($_REQUEST['name'] as $old_base64 => $new_name) {
			if($old_base64 == 'new' && !$new_name) {
				continue;
			}
			$old_name = base64_decode($old_base64);
			if(!$new_name || $new_name != $old_name) {
				$group = new usergroup_nacoma_Model($old_name);
				$group->lazy_write = true;
				$group->banish_object();
				if(!$new_name) {
					$users = self::modify_user_memberships($users, $old_name);
					$removed[] = $old_name;
					continue;
				}
			}
			$group = new usergroup_nacoma_Model($new_name);
			$old_perm_array = isset($group->obj['access_rights']) ? $group->obj['access_rights'] : array();
			$group->lazy_write = true;
			$group->obj['access_rights'] = isset($_REQUEST['access_right'][$old_base64]) ? array_keys($_REQUEST['access_right'][$old_base64]) : array();
			if($group->save_object()) {
				$group_diff = count(array_diff($group->obj['access_rights'], $old_perm_array)) + count(array_diff($old_perm_array, $group->obj['access_rights']));
				if ($group_diff > 0 || $old_name != $new_name || $old_base64 == 'new') {
					if ($old_name !== $new_name) {
						$users = self::modify_user_memberships($users, $old_name, $new_name);
					}
					$saved[] = "<strong>".$new_name."</strong>".(($old_name != $new_name) && $old_base64 != 'new' ? " (previously <strong>$old_name</strong>)" : null);
				}
			} else {
				$this->flash("<img src='images/icons/shield-critical_16x16.png' />".key($group->validation_error_list).": ".current($group->validation_error_list));
			}
		}
		if($removed) {
			$this->flash("<img src='images/icons/shield-ok_16x16.png' />Successfully removed <strong>".implode(", ", $removed)."</strong>");
		}
		if($saved) {
			$this->flash("<img src='images/icons/shield-ok_16x16.png' />Successfully saved group rights for <strong>".implode(", ", $saved)."</strong>");
		}
		usergroup_nacoma_Model::persist();
		Op5Config::instance()->setConfig('auth_users', $users);
		return self::redirect($this, 'index');
	}
}
