<?php
require_once('config.inc.php');
require_once('changelog.inc.php');
require_once('autoscan.inc.php');

function get_object_index()
{
	$object_index = array();
	global $Supported_Object_Types;
	foreach ($Supported_Object_Types as $obj_type) {
		$query = false;
		$db_table = $obj_type;
		switch ($obj_type) {
		case 'FILE':
			$db_table = 'file_tbl';
			# fallthrough
		case 'host': case 'command': case 'contact': case 'timeperiod':
		case 'servicegroup': case 'hostgroup': case 'contactgroup':
			$query = "SELECT id, ${obj_type}_name FROM $db_table " .
				"WHERE ${obj_type}_name IS NOT NULL ";
			break;
		case 'service':
			$query = "(SELECT service.id, " .
				"concat(concat(host.host_name, ';'), service.service_description) " .
				"AS service_name " .
				"FROM host, service " .
				"WHERE host.id = service.host_name " .
				"AND service.service_description IS NOT NULL " .
				") UNION ALL" .
				"(SELECT service.id, " .
				"concat(concat(hostgroup.hostgroup_name, ';'), service.service_description) " .
				"AS service_name " .
				"FROM hostgroup, service " .
				"WHERE hostgroup.id = service.hostgroup_name " .
				"AND service.service_description IS NOT NULL " .
				")";
			break;
		default:
			continue 2; # objects without incoming references can be ignored
		}

		$result = nacoma_db::exec_query($query);

		# return empty array if there were no objects to find
		$object_index[$obj_type] = array();
		while($row = $result->fetch(PDO::FETCH_NUM)) {
			$object_index[$obj_type][$row[0]] = $row[1];
		}

		# fetch templates for relevant types:
		if (in_array($obj_type, array("host", "service", "contact"))) {
			$query = "SELECT id, name FROM $db_table WHERE name IS NOT NULL";
			$result = nacoma_db::exec_query($query);
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				$object_index[$obj_type . '_template'][$row[0]] = $row[1];
			}
		}
	}

	return $object_index;
}

/**
 * Writes objects to configuration files in a format that
 * Nagios can read.
 * @param $obj_list Array with list of objects to write
 */
function glue_export($obj_list)
{
	$buf = '';

	$buf .= "###############################################################################\n";
	$buf .= "#  Generated by op5 Monitor webconfiguration exporter\n#\n";
	$buf .= "#  Exported " . date("Y-m-d H:i") . " by " .
		get_user() . "\n";
	$buf .= "#\n\n";
	$last_host = false;
	$last_hostgroup = false;
	foreach($obj_list as $obj_key => $obj) {
		$obj_type = $obj['obj_type'];

		# this shouldn't happen, but with clumsy callers
		# it might, so just skip those meta-objects
		if ($obj_type === 'FILE')
			continue;

		unset($obj['obj_type']);
		$p = get_object_properties($obj_type);

		$buf .= "\n";

		if($obj_type === 'service' &&
		   isset($obj['host_name']) &&
		   $last_host !== $obj['host_name']) {
			$last_host = $obj['host_name'];
			$last_hostgroup = false;
			$buf .= "####################################################\n" .
			  "#\n# Services for host $obj[host_name]\n#\n\n";
		}
		elseif($obj_type === 'service' &&
		       isset($obj['hostgroup_name']) &&
		       $last_hostgroup !== $obj['hostgroup_name']) {
			$last_hostgroup = $obj['hostgroup_name'];
			$last_host = false;
			$buf .= "####################################################\n" .
			  "#\n# Services for hostgroup $obj[hostgroup_name]\n#\n\n";
		}
		$buf .= "# $obj_type ";
		if(isset($obj['name'])) {
			$buf .= "template '$obj[name]'";
		}
		elseif(isset($obj['dependent_service_description']) && isset($obj['dependent_host_name']))  {
			$buf .= "for service '$obj[dependent_service_description]' on host '$obj[dependent_host_name]'";
		}
		elseif(isset($obj['dependent_service_description']) && isset($obj['dependent_hostgroup_name']))  {
			$buf .= "for service '$obj[dependent_service_description]' on hostgroup '$obj[dependent_hostgroup_name]'";
		}
		elseif(isset($obj['service_description']) && isset($obj['host_name']) && $obj_type !== 'service') {
			$buf .= "for service '$obj[service_description]' on host '$obj[host_name]'";
		}
		elseif(isset($obj['service_description']) && isset($obj['hostgroup_name']) && $obj_type !== 'service') {
			$buf .= "for service '$obj[service_description]' on hostgroup '$obj[hostgroup_name]'";
		}
		elseif(isset($obj['dependent_host_name'])) {
			$buf .= "for host '$obj[dependent_host_name]'";
		}
		elseif(isset($obj['dependent_hostgroup_name'])) {
			$buf .= "for hostgroup '$obj[dependent_hostgroup_name]'";
		}
		elseif($obj_type !== 'host' && isset($obj['host_name']) && $obj_type !== 'service') {
			$buf .= "for host '$obj[host_name]'";
		}
		elseif($obj_type !== 'hostgroup' && isset($obj['hostgroup_name']) && $obj_type !== 'service') {
			$buf .= "for hostgroup '$obj[hostgroup_name]'";
		}
		elseif(isset($p['topic']) && isset($obj[$p['topic']])) {
			$buf .= "'" . $obj[$p['topic']] . "'";
		}
		$buf .= "\ndefine $obj_type{\n";

		// fix service slave objects pretty printing, and make sure templates
		// always goes to the top.
		$v_index = array('use' => false);
		if(isset($p['var_index']['service'])) {
			$v_index['host_name'] = false;
			$v_index['hostgroup_name'] = false;
			$v_index['service_description'] = false;
		}
		if(isset($p['var_index']['dependent_service'])) {
			$v_index['dependent_host_name'] = false;
			$v_index['dependent_hostgroup_name'] = false;
			$v_index['dependent_service_description'] = false;
		}
		// these have different names in nagios compared to in nacoma.
		if($obj_type == 'contact') {
			$v_index['service_notification_commands'] = false;
			$v_index['host_notification_commands'] = false;
		}
		# done as two loops, as it results in custom variables being written
		# at the end, which looks better
		$v_index = array_merge(array_keys($v_index), array_keys($p['var_index']));
		foreach($v_index as $k) {
			if (!isset($obj[$k]) || $k === 'FILE')
				continue;
			$buf .= sprintf("    %-30s %s\n", $k, $obj[$k]);
			unset($obj[$k]);
		}
		foreach($obj as $k => $v) {
			if ($k === 'FILE')
				continue;
			$buf .= sprintf("    %-30s %s\n", $k, $v);
		}

		$buf .= "    }\n";
	}

	return $buf;
}

/**
 * Check if an array of files are writable or not
 * @param $fnames Array of filenames to check
 * @return 0 if all paths are writable, array of failed files otherwise.
 */
function unwritable_paths($fnames)
{
	global $HOME;
	$non_writable = array();
	foreach ($fnames as $f) {
		if (!posix_access($HOME.$f, POSIX_W_OK) && !posix_access($f))
			$non_writable[] = $f;
	}

	return $non_writable;
}

/**
 * Sorts $obj_array into a new array based on which file
 * each object should go in.
 * @return false if any object lacks a 'FILE' variable. Array otherwise.
 */
function obj_array_file_sort($obj_array, $changed_files)
{
	$f_array = array();
	// if the change renders a file empty, we must still set the array key
	foreach ($changed_files as $file)
		$f_array[$file] = array();

	foreach ($obj_array as $obj_type => $obj_list) {
		if (empty($obj_list)) {
			continue;
		}

		foreach($obj_list as $obj_key => $obj) {
			// no file to write object, critical fail
			if (!isset($obj['file_id']))
				return false;
			// file to write object isn't in changed files, critical fail
			if (!isset($f_array[$obj['file_id']]))
				return false;

			// temporarily set object type inside object
			$obj['obj_type'] = $obj_type;
			if (isset($obj['register']) && $obj['register'] === '1') {
				unset($obj['register']);
			}
			if (isset($obj['members']))
				unset($obj['members']);

			if (isset($obj['template'])) {
				$obj['use'] = $obj['template'];
				unset($obj['template']);
			}

			// we needed to rename these to fit them in the database.
			// rename them back again.
			if ($obj_type == 'contact') {
				if (isset($obj['service_notification_cmds'])) {
					$obj['service_notification_commands'] = $obj['service_notification_cmds'];
					unset($obj['service_notification_cmds']);
				}
				if (isset($obj['host_notification_cmds'])) {
					$obj['host_notification_commands'] = $obj['host_notification_cmds'];
					unset($obj['host_notification_cmds']);
				}
			}

			$f = $obj['file_id'];
			unset($obj['file_id']);

			$f_array[$f][] = $obj;
			unset($obj_array[$obj_type][$obj_key]);
		}
	}
	return $f_array;
}

/**
 * Fetches all objects of the selected type
 * It takes an array of files, and only retrieves
 * objects that goes into those files.
 */
function get_objects($obj_type, $for_files) {
	global $DEBUG;

	$table = $obj_type;
	$order = " ORDER BY register";
	$where = false;
	# do some extra fiddling to get sorting right
	switch ($obj_type) {
	 case 'host_template':
		$table = 'host';
		$where = " register = 0 AND host_name IS NULL";
		# fallthrough
	 case 'host':
		$order .= ', name, host_name';
		break;

	 case 'service_template':
		$table = 'service';
		$where = " register = 0 AND host_name IS NULL AND service_description IS NULL";
		# fallthrough
	 case 'service':
		$order .= ', name, host_name, service_description';
		break;

	 case 'hostdependency':
		$order .= ', dependent_host_name, host_name';
		break;

	 case 'servicedependency':
		$order .= ', dependent_service, service';
		break;

	 case 'hostescalation':
		$order .= ', host_name';
		break;

	 case 'serviceescalation':
		$order .= ', service';
		break;

	 case 'command':
		$order = ' ORDER BY command_name';
		break;

	 case 'contact_template':
		$table = 'contact';
		$order .= ', name';
		$where = ' contact_name = 0';
		break;

	 case 'timeperiod': case 'hostgroup': case 'servicegroup':
	 case 'contact': case 'contactgroup':
		$order .= ', ' . $obj_type . '_name';
		break;

	 case 'FILE':
		$table = 'file_tbl';
		$order = '';
		break;

	 default:
		$order = "";
		break;
	}

	$query = "SELECT $table.* FROM $table ";
	if ($for_files) {
		$in_str = '(';
		foreach ($for_files as $file) {
			$in_str .= nacoma_db::quote_string($file).',';
		}
		$in_str[strlen($in_str)-1] = ')';
		$query .= " INNER JOIN file_tbl f ON $table.file_id = f.id " .
		          " AND f.file_name IN $in_str ";
	}

	if ($where)
		$query .= " WHERE " . $where;
	$query .= $order;

	$result = nacoma_db::exec_query($query);
	$res = nacoma_db::fetch_result($result);
	unset($result);
	if (!empty($res)) {
		$p = get_object_properties($obj_type);
		foreach ($p['special'] as $key => $type) {
			$obj_var = $table;
			if ($p['var_index'][$key] !== 'mselect')
				continue;
			if (is_array($type))
				continue;
			$jc = get_junction_table_name($obj_var, $key);
			if($obj_type === 'host' && $key === 'children') {
				$obj_var = 'parents';
			} else if (($obj_type === 'host' && $key === 'parents') || ($obj_type === 'timeperiod' && $key === 'exclude')) {
				$type = $key;
			} else if (substr($key, -13) === 'group_members') {
				$type = 'members';
			}

			$query = "SELECT $obj_var AS id, $type FROM $jc";
			$result = nacoma_db::exec_query($query);
			while ($rel = $result->fetch(PDO::FETCH_ASSOC)) {
				if (!isset($res[$rel['id']]))
					continue;
				if (!isset($res[$rel['id']][$key]))
					$res[$rel['id']][$key] = array($rel[$type]);
				else
					$res[$rel['id']][$key][] = $rel[$type];
			}
			unset($result);
		}
	}
	return($res);
}
/**
 * Fetches all objects from all tables in the database, complete
 * with junction data.
 * @returns An array of the fetched objects.
 */
function fetch_all_objects($for_files)
{
	global $Supported_Object_Types;

	foreach($Supported_Object_Types as $obj_type) {
		if($obj_type === 'FILE') continue;
		$obj_list = get_objects($obj_type, $for_files);

		# unset 'children' variable. Nagios doesn't grok it
		if($obj_type === 'host' && is_array($obj_list)) {
			foreach($obj_list as $obj_key => $obj) {
				if(!isset($obj['children'])) continue;
				unset($obj['children']);
				$obj_list[$obj_key] = $obj;
			}
		}
		$obj_array[$obj_type] = $obj_list;
	}

	return $obj_array;
}

/**
 * Writes the by-file sorted object array to their respective
 * files. Files that were previously used and are now empty
 * are truncated to prevent stale objects with now-bogus
 * object-references from remaining.
 * @param $f_array by-file sorted object array
 */
function export_write_files($f_array)
{
	global $HOME;

	# write the stuff back to its files
	foreach($f_array as $fname => $obj_list) {
		if ($fname{0} !== '/')
			$fname = "$HOME/$fname";
		write_file($fname, glue_export($obj_list));
	}
}

/**
 * Update the changelog with time, date and name of export
 * @param $user Name of user doing the update
 */
function export_update_changelog($user = false)
{
	if (!$user)
		$user = get_user();

	nacoma_db::exec_query("INSERT INTO action_log(username, action) VALUES(" .
                       nacoma_db::quote_string($user) . ", 'export')");
}

/**
 * Helper for delete_config_backup
 * @param $path Path to delete recursively
 */
function recursive_delete($path, $keep_directory=false)
{
	$path = rtrim($path, '/');
	if (!file_exists($path))
		return false;

	if (is_file($path))
		return unlink($path);

	foreach (glob($path . '/*') as $subpath) {
		recursive_delete($subpath);
	}
	if (!$keep_directory)
		rmdir($path);
}

/**
 * Cleans up the config backup directory
 *
 * Called before each export, and after each successful export.
 */
function delete_config_backup()
{
	global $Backup_Config_Dir;
	# in case this is not set, we shouldn't delete other files.
	# that wouldn't be appreciated.
	if (!$Backup_Config_Dir)
		return;
	recursive_delete($Backup_Config_Dir, true);
}

/**
 * Backup config to a temporary directory during the export.
 */
function backup_config($files)
{
	delete_config_backup();
	global $Backup_Config_Dir, $HOME;
	foreach ($files as $file) {
		$backup_subdir = dirname($file);
		if (!@mkdir(rtrim($Backup_Config_Dir, '/') . '/' . $backup_subdir, 0777, true))
			continue;
		if ($file[0] != '/')
			$file = rtrim($HOME, '/').'/'.$file;
		if (file_exists($file))
			copy($file, rtrim($Backup_Config_Dir, '/') . '/' . $backup_subdir . '/' . basename($file));
	}
}

/**
 * Restore backed up config in case of export verification errors.
 */
function restore_backup($files)
{
	global $Backup_Config_Dir, $HOME;
	foreach ($files as $file) {
		// store a copy since the variable('s path) will be modified
		$submitted_filename = $file;
		$backup_subdir = dirname($file);
		if ($file[0] != '/') {
			$file = rtrim($HOME, '/').'/'.$file;
		}
		$directory_for_backups = rtrim($Backup_Config_Dir, '/').'/';
		if (file_exists($directory_for_backups. $submitted_filename)) {
			copy($directory_for_backups. $backup_subdir . '/' . basename($file), $file);
		}
	}
}

/**
 * Runs Nagios' configuration verification routines to ensure
 * that the new configuration is sane and can be loaded properly.
 * @return true on success, array on errors
 */
function exported_config_is_ok()
{
	global $Config;
	$cmd = '/usr/bin/asmonitor -q naemon -v '.$Config['main'];
	$verify = exec_cmd($cmd);
	return $verify['value'] === 0 ? true : $verify;
}

/**
 * Check to see if monitor/nagios is running.
 * @return true if yes, false if not
 */
function monitor_is_running()
{
	exec("asmonitor service naemon status 2>&1", $output, $ret);
	return $ret === 0;
}

/**
 * Try to reload monitor by sending a command to the command pipe
 * @return 0 on success. 1 if monitor isn't running. 2 if the naemon
 * restart command failed.
 */
function export_reload_monitor()
{
	$running = monitor_is_running();
	if($running === false)
		return 1;
	exec("sudo service naemon restart", $output, $ret);
	if($ret === 0)
		return 0;
	else
		return 2;
}


/**
 * This is the main export function used by export.php
 *
 * @param $changes A changelog object to only export those changes, or TRUE to export everything
 *
 * @return TRUE if the config was successfully exported, and an array describing the error otherwise
 */
function do_export($changes)
{
	$l = op5Log::instance('nacoma');
	$l->log('debug', 'Starting a save');
	global $Supported_Object_Types;
	global $SUPPORT, $DEBUG;
	if (!nacoma_objlock::extend_lock('all', 0, 5*60)) {
		$l->log('warning', 'Trying to save, while save is already in progress');
		return array('type' => 'save in progress');
	}
	if ($changes !== true) {
		$overlay = array(); $changed_files = array();
		$changes->undo_others_changes($overlay, $changed_files);
		if (empty($changed_files)) {
			nacoma_objlock::free_lock('all', 0);
			$l->log('debug', 'Trying to save, but nothing to save');
			return array('type' => 'nothing to do');
		}
	} else {
		$changed_files = array();
		$overlay = array();
		$res = nacoma_db::exec_query('SELECT file_name FROM file_tbl');
		while ($row = $res->fetch(PDO::FETCH_ASSOC))
			$changed_files[] = $row['file_name'];
		unset($res);
	}
	$non_writable = unwritable_paths($changed_files);
	if ($non_writable) {
		nacoma_objlock::free_lock('all', 0);
		$l->log('error', 'Trying to save, But files are unwritable: ' . implode(',', $non_writable));
		return array('type' => 'file write error', 'files' => $non_writable);
	}
	$l->log('debug', 'Starting to fetch objects for save');
	$obj_array = fetch_all_objects($changed_files);
	$l->log('debug', 'Fetching index for save');
	$object_index = get_object_index();
	$l->log('debug', 'Merging overlay for save');
	# merge the overlay with the fetched objects
	foreach ($overlay as $type => $objs) {
		if (substr($type, -strlen('_template')) == '_template')
			$type = substr($type, 0, -strlen('_template'));

		if (!isset($obj_array[$type])) {
			$obj_array[$type] = array();
		}
		foreach ($objs as $id => $obj) {
			if ($obj === false || $obj->obj === false) {
				unset($obj_array[$type][$id]);
				continue;
			}
			unset($obj->obj['id']);
			if($type === 'host' && isset($obj->obj['children']))
				unset($obj->obj['children']);
			$obj_array[$type][$id] = $obj->obj;
			$object_index[$type][$id] = $obj->get_object_name();
		}
	}

	$l->log('debug', 'Normalizing data for save');
	foreach($Supported_Object_Types as $obj_type) {
		if($obj_type === 'FILE') continue;

		// only treat objects that exists
		if(empty($obj_array[$obj_type])) {
			continue;
		}

		$l->log('debug', "Getting object properties for object type $obj_type");
		$props = get_object_properties($obj_type);
		$spec = $props['special'];
		$v_index = $props['var_index'];

		$l->log('debug', "Normalizing data for object type $obj_type");
		$obj_list = $obj_array[$obj_type];
		foreach($obj_list as $obj_key => $obj) {
			foreach($v_index as $k => $v) {
				# the 'members' directive is removed later, so
				# ignore it here or hostgroups/servicegroups
				# used with templates will complain about not
				# being able to locate some of their member objects
				if ($k === 'members')
					continue;

				// VALIDATE OBJECTS BEFORE WE GET TO THIS!!!
				if(!isset($obj[$k]))
					continue;

				// fix single references that aren't image refs.
				if(($v === 'select' && !is_array($spec[$k])) ||
				   ($v === 'hidden' && $k !== 'register')) {
					if (isset($object_index[$spec[$k]][$obj[$k]])) {
						$ref = $object_index[$spec[$k]][$obj[$k]];
					} else {
						if ($DEBUG && $obj[$k])
							echo "object_index[ $spec[$k] ][ $obj[$k] ] isn't set (obj_type is $obj_type)!<br />\n";
						unset($obj[$k]);
						continue;
					}
					if (($obj_type === 'servicedependency' || $obj_type === 'serviceescalation') && ($k === 'service' || $k === 'dependent_service')) {
						$prefix = (strpos($k, 'dependent') === 0) ? 'dependent_' : '';
						$str = explode(';', $ref, 2);
						$parent_type = service_nacoma_Model::get_parent_type($obj[$k]);
						$obj[$prefix.$parent_type.'_name'] = $str[0];
						$obj[$prefix.'service_description'] = $str[1];
						unset($obj[$k]);
					}
					else {
						$obj[$k] = $ref;
					}
					if($spec[$k] === 'command' && isset($obj[$k . '_args'])) {
						$obj[$k] .= '!' . $obj[$k . '_args'];
						unset($obj[$k . '_args']);
					}
				}
				elseif($v === 'mselect') {
					$tmp = false;
					if (is_array($obj[$k]))
						foreach($obj[$k] as $v_var) {
							if(isset($object_index[$spec[$k]][$v_var])) {
								$tmp[] = $object_index[$spec[$k]][$v_var];
							} else if ($DEBUG && $v_var) {
								echo "object_index[ $spec[$k] ][ $v_var ] isn't set (obj_type is $obj_type)!<br />\n";
							}
						}
					if(!empty($tmp))
						$obj[$k] = implode(',', $tmp);
					else
						unset($obj[$k]);
				}
				elseif($v === 'string') {
					# never export "_args" variables without
					# their corresponding command reference
					if (substr($k, -5) === '_args' && !isset($obj[substr($k, 0, -5)])) {
						unset($obj[$k]);
						continue;
					}
				}
				elseif($v === 'checkbox') {
					$obj[$k] = stringify_checkbox($obj[$k]);
				}
			}
			$custom = get_custom_vars($obj_type, $obj_key);
			if (!empty($custom)) {
				$obj = array_merge($obj, $custom);
			}
			$obj_list[$obj_key] = $obj;
		}
		$obj_array[$obj_type] = $obj_list;
	}

	$l->log('debug', 'Sorting data for save');
	$f_array = obj_array_file_sort($obj_array, $changed_files);
	if ($f_array === false) {
		nacoma_objlock::free_lock('all', 0);
		$l->log('error', 'Trying to save, but got ye olde evile FILE error');
		return array('type' => 'FILE');
	}

	$l->log('debug', 'Backing up current data before save');
	backup_config($changed_files);

	$l->log('debug', 'Writing new configuration for save');
	export_write_files($f_array);

	$l->log('debug', 'Updating changelog for save');
	export_update_changelog(get_user());

	// FIXME: This shortcut is incorrect - we need to recalculate this completely post-export
	// Validate autoscan results
	if ($changes !== true) {
		$hosts = $changes->get_host_list();
		foreach ($hosts as $host) {
			autoscan_result_visibility_change($host['address'], ($host['change'] == 'Add' ? 0 : 1));
		}
	}

	$l->log('debug', 'Verifying config in save');
	$verify = exported_config_is_ok();

	if($verify !== true) {
		restore_backup($changed_files);
		nacoma_objlock::free_lock('all', 0);
		$l->log('warning', 'Trying to save, but preflight check failed');
		return array('type' => 'preflight check failed', 'output' => $verify['buf']);
	}

	$l->log('debug', 'Configuration ok, removing config backup for save');
	delete_config_backup();

	# everything worked - let's run hooks
	require_once('hooks.inc.php');
	$l->log('debug', 'Running save hooks');
	run_hooks('save', ($changes !== true? $changes->get_raw_changes() : true));

	$l->log('debug', 'Running save denormalized hooks');
	run_hooks('save_denormalized', ($changes !== true? $changes->get_raw_changes() : true));

	$l->log('debug', 'Cleaning up changelog post save');
	if ($changes !== true) {
		$changes->delete_changes();
	} else {
		$c = new ChangeLog_nacoma_Model(null, null, true);
		$c->delete_changes();
	}

	nacoma_objlock::free_lock('all', 0);
	$l->log('debug', 'Save completed');
	return true;
}
