<?php

define('SKIP_KOHANA', true);
require('/opt/monitor/op5/ninja/index.php');

include_once('include/webconfig.inc.php');
include_once('classes/common_gui.php');
include_once('include/nacoma_access.inc.php');
require_once('op5/log.php');

$log = Op5log::instance('logger');

validate_user();

$access = get_nacoma_access('logger_configuration');
if (!in_array(NACOMA_ACCESS_FULL_ACCESS, $access)) {
	deny_access("You need 'logger_configuration' permissions to modify Logger Settings");
}

try {
	$db = new PDO('pgsql:host=localhost;dbname=op5lsu', 'op5lsu', 'op5lsu');
} catch(PDOException $e) {
	deny_access("Error: Could not connect to logger database.");
}
$default_config = array(
	"keep_in_db" => "192",
	"storage_path" => ""
);
$config = array_merge(
	$default_config,
	$db->query("SELECT ".implode(', ', array_keys($default_config))." FROM settings")->fetch(PDO::FETCH_ASSOC)
);
$keep_in_db_value = $config["keep_in_db"];
$storage_path_value = $config["storage_path"];


if (isset($_REQUEST['action'])) {
	$errors = array();
	$intmatch = '/^[0-9]+$/';
	if (!preg_match($intmatch, $_REQUEST['keep_in_db'])) {
		$errors['keep_in_db'] = "'Keep in database for' must be a positive whole number";
	}
	if (!empty($_REQUEST['storage_path'])) {
		if (!is_dir($_REQUEST['storage_path'])) {
			$errors['storage_path'] = "'Storage path' must be a directory";
		}
		else if (realpath($_REQUEST['storage_path']) != $_REQUEST['storage_path']) {
			$errors['storage_path'] = "'Storage path' must be an absolute path";
		}
	}

	if (!$errors) {
		$result = $db->prepare("UPDATE settings
				SET
				keep_in_db = :keep_in_db,
				storage_path = :storage_path")
				->execute(array(
					':keep_in_db' => $_REQUEST['keep_in_db'],
					':storage_path' => $_REQUEST['storage_path']
				));
		if (!$result) {
			$errors['db_query'] = "Failed to execute query";
		}
	}
	if($errors) {
		$_SESSION['ls_errors'] = $errors;
	} else {
		$log->log('debug', get_user() . ' changed the Logger Settings');
		$_SESSION['ls_successful'] = 1;
	}
	header("Location: ".$_SERVER['REQUEST_URI']);
	exit;
}

$errors = isset($_SESSION['ls_errors']) ? $_SESSION['ls_errors'] : array();
unset($_SESSION['ls_errors']);

$page->html_start('Logger Configuration');

if ($errors) {
	echo '<div id=errors><p>';
	foreach ($errors as $message) {
		echo $message . '<br />';
	}
	echo '</p></div>';
}

if(isset($_SESSION['ls_successful'])) {
	print '<p><img src="images/icons/shield-ok_16x16.png" alt="" style="margin-bottom: -3px" /> <strong>Successfully updated settings</strong></p>';
	unset($_SESSION['ls_successful']);
}
// Form rendering
$page->form_start();
?>
<table class="form-table">
	<tr>
		<td>
			<img src='images/icons/shield-help.png' class='ls_config_help' style='cursor: pointer;' data-help='keep_in_db' alt='Keep in database for' title='Rotate database' />
		</td>
		<td>
			<label for=keep_in_db>Keep in database for</label>
		</td>
		<td>
			<input id=keep_in_db name='keep_in_db' type='text' value="<?php echo htmlspecialchars($keep_in_db_value) ?>" <?php echo isset($errors['keep_in_db']) ? "class='input_error'" : "" ?> /> <small>hours</small>
			<?php echo isset($errors['keep_in_db']) ? "<br />" . htmlspecialchars($errors['keep_in_db']) : "" ?>
		</td>
	</tr>
	<tr>
		<td>
			<img src='images/icons/shield-help.png' class='ls_config_help' style='cursor: pointer;' data-help='archive_logs' />
		</td>
		<td>
			<label for="enable_archive">Archive logs</label>
		</td>
		<td>
			<input id='enable_archive' type='checkbox' <?php echo empty($storage_path_value)?"":'checked="checked"'; ?> /> <label for="enable_archive">Archive logs as text files</label>
		</td>
	</tr>
	<tr>
		<td>
			<img src='images/icons/shield-help.png' class='ls_config_help' style='cursor: pointer;' data-help='storage_path' alt='Storage path' title='Storage path' />
		</td>
		<td>
			<label for=storage_path>Storage path</label>
		</td>
		<td>
			<input id=storage_path name='storage_path' type='text' value="<?php echo htmlspecialchars($storage_path_value) ?>" <?php echo isset($errors['storage_path']) ? "class='input_error'" : "" ?> />
			<?php echo isset($errors['storage_path']) ? "<br />" . htmlspecialchars($errors['storage_path']) : "" ?>
		</td>
	</tr>
</table>
<?php
echo "<div class='max magic'>\n";
$page->submit_button("Save");
echo '</div>';
$page->form_end();
$page->html_end();
