-- Names of old tables. They only existed in versions where we didn't support
-- multiple roles with different memberships.
DROP TABLE IF EXISTS roles;
DROP TABLE IF EXISTS role_users;

--
-- Structure for table `roles`
--

CREATE TABLE IF NOT EXISTS role (
  id int(11) PRIMARY KEY AUTO_INCREMENT,
  role_name varchar(255)
);

INSERT INTO role (id, role_name) VALUES (1, 'default') ON DUPLICATE KEY UPDATE role_name=role_name;

--
-- Structure for table `username_role`
--

-- This should handle upgrades on our internal beta systems without errors.
-- It will kill all their old beta roles, but I don't care about that.
-- Real users don't have real roles yet.
DROP TABLE IF EXISTS user_role;

CREATE TABLE IF NOT EXISTS username_role (
  username varchar(255) NOT NULL,
  role int(11) NOT NULL,
  FOREIGN KEY (role) REFERENCES role(id),
  INDEX (username)
) COLLATE latin1_general_cs;


--
-- Structure for table `role_permissions`
--
-- (this should create an error if it already exists, which will error out
-- before the inserts)
CREATE TABLE role_permissions (
  id int(11) PRIMARY KEY AUTO_INCREMENT,
  role_id int(11) NOT NULL,
  object_type varchar(100) NOT NULL,
  object_field varchar(100) DEFAULT NULL,
  rule text NOT NULL,
  FOREIGN KEY (role_id) REFERENCES role(id)
) COLLATE latin1_general_cs;


-- If tables already existed, we'll have errored out by now
INSERT INTO role_permissions (role_id, object_type, rule)
  VALUES (1, 'host', 'allow'),
         (1, 'service', 'allow'),
         (1, 'contact', 'allow'),
         (1, 'hostgroup', 'allow'),
         (1, 'servicegroup', 'allow'),
         (1, 'contactgroup', 'allow'),
         (1, 'template', 'allow'),
         (1, 'host_template', 'allow'),
         (1, 'service_template', 'allow'),
         (1, 'contact_template', 'allow'),
         (1, 'command', 'allow'),
         (1, 'timeperiod', 'allow'),
         (1, 'FILE', 'allow'),
         (1, 'servicedependency', 'allow'),
         (1, 'serviceescalation', 'allow'),
         (1, 'hostdependency', 'allow'),
         (1, 'hostescalation', 'allow'),
         (1, 'pnp', 'allow');
