<?php

/**
 * Naemon objects share a lot of behavior that non-naemon objects don't share.
 *
 * Ideally, this would have been in place from the beginning, so
 * object_nacoma_Model would have contain the most generic implementations of
 * all methods (or no implementation at all), and then left it up to
 * this to override those implementations where it makes sense.
 *
 * Alas, Voltaire was full of shit.
 */
abstract class naemon_nacoma_Model extends object_nacoma_Model
{
	public static $objects_for_user = array();

	public function save_object($user=true)
	{
		unset(self::$objects_for_user[static::TYPE]);
		return parent::save_object($user);
	}

	public function delete_object()
	{
		unset(self::$objects_for_user[static::TYPE]);
		return parent::delete_object();
	}

	public function banish_object($user=true)
	{
		unset(self::$objects_for_user[static::TYPE]);
		return parent::banish_object($user);
	}

	static private function get_user_object_index_table() {
		$obj_type = static::TYPE;
		$exclude_pending_query = 'id '.get_exclude_pending_string($obj_type);

		$query = false;
		$db_table = $obj_type;
		switch ($obj_type) {
		case 'FILE':
			$db_table = 'file_tbl';
			# fallthrough
		case 'host': case 'command': case 'contact': case 'timeperiod':
		case 'servicegroup': case 'hostgroup': case 'contactgroup':
			$query = "SELECT id, ${obj_type}_name FROM $db_table " .
				"WHERE ${obj_type}_name IS NOT NULL " .
				"AND $exclude_pending_query " .
				"ORDER BY ${obj_type}_name";
			break;
		case 'host_template': case 'contact_template': case 'service_template':
			$query = "SELECT id, name " .
				"FROM " . substr($obj_type, 0, -strlen('_template')) . " " .
				"WHERE name IS NOT NULL " .
				"AND $exclude_pending_query " .
				"ORDER BY name";
			break;
		case 'service':
			$query = "(SELECT service.id, " .
				"concat(concat(host.host_name, ';'), service.service_description) " .
				"AS service_name " .
				"FROM host, service " .
				"WHERE host.id = service.host_name " .
				"AND service.service_description IS NOT NULL " .
				"AND service.$exclude_pending_query " .
				") UNION ALL" .
				"(SELECT service.id, " .
				"concat(concat(hostgroup.hostgroup_name, ';'), service.service_description) " .
				"AS service_name " .
				"FROM hostgroup, service " .
				"WHERE hostgroup.id = service.hostgroup_name " .
				"AND service.service_description IS NOT NULL " .
				"AND service.$exclude_pending_query " .
				") ORDER BY service_name";
			break;
		case 'hostdependency': case 'hostescalation':
		case 'serviceescalation': case 'servicedependency':
			$query = "SELECT id FROM $obj_type WHERE " . $exclude_pending_query;
			$result = nacoma_db::exec_query($query);
			$res = array();
			while($row = $result->fetch(PDO::FETCH_NUM)) {
				$res[$row[0]] = get_object_name_by_id($obj_type, $row[0]);
			}
			return $res;
			break;
		default:
			return false;
		}

		$result = nacoma_db::exec_query($query);
		if (!$result)
			return(false);

		# return empty array if there were no objects to find
		$res = array();
		while($row = $result->fetch(PDO::FETCH_NUM)) {
			$res[$row[0]] = $row[1];
		}

		return $res;
	}

	static private function get_auth_objects_contact_access()
	{
		$user = nacoma_db::quote_string(get_user());
		$get_template_tree = function($objname, $type) {
			$objects = array();
			$new_objects = nacoma_db::exec_query("
			SELECT
				id, {$type}_name
			FROM
				{$type}
			WHERE
				{$type}_name = ".$objname." AND {$type}.id ".get_exclude_pending_string($type))
			->fetchAll(PDO::FETCH_KEY_PAIR);
			$safety_limit_depth = 30; // because while-true looks so guilty
			while($new_objects && --$safety_limit_depth) {
				foreach($new_objects as $key => $val)
					$objects[$key] = $val;
				$new_objects = nacoma_db::exec_query("
				SELECT
					tmpl.id, tmpl.name
				FROM
					{$type} AS obj
				INNER JOIN
					{$type} AS tmpl
					 ON obj.template = tmpl.id
				WHERE
					obj.id IN (".implode(", ", array_keys($new_objects)).") AND tmpl.id ".get_exclude_pending_string($type))
					->fetchAll(PDO::FETCH_KEY_PAIR);
				$new_objects = array_diff($new_objects, $objects);
			}
			return $objects;
		};
		$get_all_objects = function($cgs, $cs, $type) {
			if ($type == 'service') {
				$name = 'concat(concat(ifnull(host.host_name, hostgroup.hostgroup_name), \';\'), obj.service_description)';
			} else {
				$name = 'obj.'.$type.'_name';
			}
			$query = "
			SELECT
				obj.id, $name AS {$type}_name, obj.name
			FROM
				{$type} as obj
			LEFT JOIN
				{$type}_contact
				ON {$type}_contact.$type = obj.id
			LEFT JOIN
				{$type}_contactgroup
				ON {$type}_contactgroup.$type = obj.id";
			if ($type == 'service')
				$query .= "
				LEFT JOIN host ON host.id = obj.host_name
				LEFT JOIN hostgroup ON hostgroup.id = obj.hostgroup_name";
			$query .= "
			WHERE
				".(!empty($cs)?"{$type}_contact.contact IN (".implode(', ', array_keys($cs)).")":"0=1")."
			OR
				".(!empty($cgs)?"{$type}_contactgroup.contactgroup IN (".implode(', ', array_keys($cgs)).")":"0=1");
			$new_objects = nacoma_db::exec_query($query)->fetchAll(PDO::FETCH_NUM);
			$all_objects = array();
			$safety_limit_depth = 30; // because while-true looks so guilty
			while($new_objects && --$safety_limit_depth) {
				$templates = array();
				foreach ($new_objects as $ary) {
					if ($ary[2]) {
						$templates[] = $ary[0];
						$all_objects[$ary[0]] = $ary[2];
					}
					else {
						$all_objects[$ary[0]] = $ary[1];
					}
				}
				if (!$templates)
					break;
				$query = "
				SELECT
					obj.id, $name AS {$type}_name, obj.name
				FROM
					{$type} AS obj";
				if ($type == 'service')
					$query .= "
					LEFT JOIN host ON host.id = obj.host_name
					LEFT JOIN hostgroup ON hostgroup.id = obj.hostgroup_name";
				$query .= "
				WHERE
					obj.template IN (".implode(", ", $templates).")
					AND obj.id ".get_exclude_pending_string($type);
				$new_objects = nacoma_db::exec_query($query)->fetchAll(PDO::FETCH_NUM);
			}
			return $all_objects;
		};
		$get_group_objects = function($type, $plains) {
			$objects = array();
			$new_objects = nacoma_db::exec_query("
			SELECT
				${type}group.id, ${type}group.${type}group_name
			FROM
				${type}group
			INNER JOIN
				${type}_${type}group ON ${type}_${type}group.${type}group = ${type}group.id
			WHERE
				".(!empty($plains)?"${type}_${type}group.${type} IN (".implode(', ', array_keys($plains)).')':'0=1')."
				AND ${type}group.id ".get_exclude_pending_string("${type}group"))
			->fetchAll(PDO::FETCH_KEY_PAIR);
			$safety_limit_depth = 30; // because while-true looks so guilty
			while($new_objects && --$safety_limit_depth) {
				foreach($new_objects as $key => $val)
					$objects[$key] = $val;
				$new_objects = nacoma_db::exec_query("
				SELECT
					${type}group.id, ${type}group.${type}group_name
				FROM
					${type}group
				INNER JOIN
					${type}group_members ON ${type}group_members.${type}group = ${type}group.id
				WHERE
					${type}group_members.members IN (".implode(', ', array_keys($new_objects)).")
					AND ${type}group.id ".get_exclude_pending_string("${type}group"))
					->fetchAll(PDO::FETCH_KEY_PAIR);
				$new_objects = array_diff($new_objects, $objects);
			}
			return $objects;
		};
		$contacts = $get_template_tree($user, 'contact');
		$cgs_for_user = $get_group_objects('contact', $contacts);

		switch (static::TYPE) {
		case 'host':
			$res = $get_all_objects($cgs_for_user, $contacts, 'host');
			return $res;
		case 'service':
			$tmp = $get_all_objects($cgs_for_user, $contacts, 'service');
			$hosts = host_nacoma_Model::get_objects_for_user();
			$hostgroups = hostgroup_nacoma_Model::get_objects_for_user();
			$res = nacoma_db::exec_query("
			SELECT
				s.id, concat(concat(ifnull(host.host_name, hostgroup.hostgroup_name), ';'), s.service_description)
			FROM
				service as s
			LEFT JOIN host ON host.id = s.host_name
			LEFT JOIN hostgroup ON hostgroup.id = s.hostgroup_name
			WHERE
				".(!empty($hosts)?"s.host_name IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($hostgroups)?"s.hostgroup_name IN (".implode(', ', array_keys($hostgroups)).")":"0=1")."
			AND s.id ".get_exclude_pending_string('service'))
				->fetchAll(PDO::FETCH_KEY_PAIR);
			foreach ($tmp as $k => $v)
				$res[$k] = $v;
			return $res;
		case 'contact':
			// The 5.4+ behaviour was to retrieve any contact on a host or service
			// I could access - keep doing that, but add anything I'm in.
			$services = service_nacoma_Model::get_objects_for_user();
			$hosts = host_nacoma_Model::get_objects_for_user();
			$res = nacoma_db::exec_query("
			SELECT
				c.id, c.contact_name
			FROM
				contact c
			LEFT JOIN host_contact hc ON hc.contact = c.id LEFT JOIN host h ON h.id = hc.host
			LEFT JOIN service_contact sc ON sc.contact = c.id LEFT JOIN service s ON s.id = sc.service
			WHERE
				".(!empty($hosts)?"h.id IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($services)?"s.id IN (".implode(', ', array_keys($services)).")":"0=1"))
				->fetchAll(PDO::FETCH_KEY_PAIR);
			foreach ($contacts as $k => $v)
				$res[$k] = $v;
			return $res;
		case 'hostgroup':
			$hosts = host_nacoma_Model::get_objects_for_user();
			return $get_group_objects('host', $hosts);
		case 'servicegroup':
			$services = service_nacoma_Model::get_objects_for_user();
			return $get_group_objects('service', $services);
		case 'contactgroup':
			// The 5.4+ behaviour was to retrieve any contactgroup on a host or service
			// I could access - keep doing that, but add anything I'm in.
			$services = service_nacoma_Model::get_objects_for_user();
			$hosts = host_nacoma_Model::get_objects_for_user();
			$res = nacoma_db::exec_query("
			SELECT
				c.id, c.contactgroup_name
			FROM
				contactgroup c
			LEFT JOIN host_contactgroup hc ON hc.contactgroup = c.id LEFT JOIN host h ON h.id = hc.host
			LEFT JOIN service_contactgroup sc ON sc.contactgroup = c.id LEFT JOIN service s ON s.id = sc.service
			WHERE
				".(!empty($hosts)?"h.id IN (".implode(', ', array_keys($hosts)).")":"0=1")."
			OR
				".(!empty($services)?"s.id IN (".implode(', ', array_keys($services)).")":"0=1"))
				->fetchAll(PDO::FETCH_KEY_PAIR);
			foreach ($cgs_for_user as $k => $v)
				$res[$k] = $v;
			return $res;
		default:
			return array();
		}

		$result = nacoma_db::exec_query($query);
		if (!$result)
			return array();

		// fetchAll() might return false..
		$result = $result->fetchAll(PDO::FETCH_KEY_PAIR);

		return $result ? $result : array();
	}

	/**
	 * Return user's objects of this type
	 *
	 * Do not call this method. Seriously, don't.
	 */
	static public function get_objects_for_user($write = false)
	{
		if (isset(self::$objects_for_user[static::TYPE][$write]))
			return self::$objects_for_user[static::TYPE][$write];

		$rules = (array)get_nacoma_access(static::TYPE);

		// Look for recognized rules, from highest-access to lowest
		if (in_array(NACOMA_ACCESS_FULL_ACCESS, $rules) ||
			(!$write && in_array(NACOMA_ACCESS_VIEW_ALL, $rules)) ||
			($write && in_array(NACOMA_ACCESS_EDIT_ALL, $rules)))
		{
			self::$objects_for_user[static::TYPE][$write] =
				static::get_user_object_index_table();
			return self::$objects_for_user[static::TYPE][$write];
		}
		else if ((!$write && in_array(NACOMA_ACCESS_VIEW_CONTACT, $rules) &&
				  !in_array(NACOMA_ACCESS_VIEW_ALL, $rules)) ||
				 ($write && in_array(NACOMA_ACCESS_EDIT_CONTACT, $rules) &&
				  !in_array(NACOMA_ACCESS_EDIT_ALL, $rules)))
		{
			// Get a list of the objects a user can access
			self::$objects_for_user[static::TYPE][$write] =
				static::get_auth_objects_contact_access();
			return self::$objects_for_user[static::TYPE][$write];
		}
		else {
			// Default action: deny
			global $DEBUG;
			if ($DEBUG && !empty($rules)) {
				ob_start();
				?>
				<p>
					The rules <?php echo implode(',', $rules); ?> supposedly
					apply to <?php echo html::specialchars(get_user()); ?> for
					objects of type <?php echo static::TYPE; ?>,
					which I don't understand.
				</p>
				<?php
				echo ob_get_clean();
			}
			return array();
		}
		return self::$objects_for_user[static::TYPE][$write];
	}
}
