<?php

/**
 * Use a service dependency when you want a service's status to depend
 * on other service(s) status(es).
 */
class servicedependency_nacoma_Model extends naemon_nacoma_Model
{
	/** object type */
	const TYPE = 'servicedependency';
	/** master object type */
	var $master_obj_type 		= 'service';
	/** Variable in this object identifying the master object */
	var $master_obj_identifier 	= 'dependent_service';

	public $properties = array (
		'topic' => NULL,
		'var_index' => array (
			'service' => 'select',
			'dependent_service' => 'select',
			'dependency_period' => 'select',
			'execution_failure_criteria' => 'checkbox',
			'notification_failure_criteria' => 'checkbox',
			'inherits_parent' => 'bool',
			'file_id' => 'select',
			'register' => 'bool'
		),
		'widget' => array (
			'dependent_service' => 'hidden',
			'register' => 'hidden'
		),
		'required' => array (
			'service',
			'dependent_service',
			'file_id'
		),
		'default' => array (
			'execution_failure_criteria' => array (
				'c',
				'w'
			),
			'notification_failure_criteria' => array (
				'c',
				'w'
			),
			'inherits_parent' => '1',
			'file_id' => 'etc/servicedependencies.cfg'
		),
		'special' => array (
			'service' => 'service',
			'dependent_service' => 'service',
			'execution_failure_criteria' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'o' => 'OK',
				'n' => false
			),
			'notification_failure_criteria' => array (
				'c' => 'Critical',
				'w' => 'Warning',
				'u' => 'Unknown',
				'o' => 'OK',
				'n' => false
			),
			'dependency_period' => 'timeperiod',
			'file_id' => 'FILE'
		)
	);

	/**
	 * Return the mayi resource for the object
	 * @see object_nacoma_Model::mayi_resource()
	 */
	public function mayi_resource() {
		return "monitor.monitoring.servicedependencies";
	}

	/**
	 * Make sure user-input is sane prior to invoking save/glue_objects
	 * @return bool: True if object validates, otherwise false
	 */
	function validate_object()
	{
		global $DEBUG;
		if (!parent::validate_object()) {
			return false;
		}
		if($this->obj['service'] === $this->obj['dependent_service']) {
			return $this->validation_error('impossible dependency', "A service can not depend on itself");
		}
		if(!empty($this->obj) && $this->id==='new') {
			$this->obj['dependent_service'] = $this->master_obj_id;
			$query 	= "SELECT service FROM ".$this->type." WHERE dependent_service = ".$this->master_obj_id;
			$result = nacoma_db::exec_query($query);
			foreach ($result->fetchAll(PDO::FETCH_NUM) as $row) {
				if($row[0] == $this->obj['service']) {
					# found a duplicate dependency, so remove it
					if($DEBUG) {
						echo "Duplicate " . $this->type . ".<br />\n";
						echo "<pre>\n"; print_r($this->obj); echo "</pre>\n";
					}

					return $this->validation_error('duplicate object', "A dependency between ".get_object_name_by_id('service', $this->obj['dependent_service'])." and ".get_object_name_by_id('service', $this->obj['service'])." already exists");
					break;
				}
			}
		}
		return true;
	}

	/**
	 * Compute the master object id
	 * @return False on errors, the object id on success
	 */
	function compute_master_obj_id()
	{
		if (!$this->master_obj_type || !$this->obj)
		  return false;

		return $this->master_obj_id;
	}

	public function get_cmd_bar_links()
	{
		$cmd_links = parent::get_cmd_bar_links();
		// instead of duplicating validation from parent, just check if the validation ended up allowing copying, and only then add clone
		if (isset($cmd_links['copy']))
			$cmd_links['clone'] = '<a title="Clone service dependencies" href="edit_special.php/copy_slaves?obj_type='.$this->type.'&amp;master_obj_id='.$this->master_obj_id.'&amp;master_obj_type='.$this->master_obj_type.'&amp;obj_id='.$this->id.'">';
		return $cmd_links;
	}

	function get_object_by_name($name)
	{
		$table = self::get_objects_for_user();
		$this->id = array_search($name, $table);
		return $this->get_object();
	}
}
