<?php

/**
 * Widget for printing a dropdown list for single object select
 */
class Select_Nacomawidget extends Base_Nacomawidget {
	/**
	 * Makes a selection of the items in the indexed array $tokenvalues, using $dflt
	 * as default selection, with a(form) 'name' of $select_name */
	public static function make_selection($tokenvalues, $dflt, $select_name = 'selection', $class_name = 'xlarge', $required = false) {
		global $master_obj; # host and servicedependencies set this
		global $DEBUG;

		/* if array is empty, we tell the user so */
		if(is_array($tokenvalues) && empty($tokenvalues)) {
			echo "None configured\n";
			return(true);
		}

		$default = "";
		if(is_array($tokenvalues)) {
			printf('<select class="'.$class_name.'" id="%1$s" name="%1$s" %sdata-filterable="true">', $select_name, $required ? ' data-required="true"' : '');
			foreach ($tokenvalues as $k => $v) {
				printf('<option value="%s"%s>%s</option>', html::specialchars($v), $v == $dflt ? ' selected="selected"' : '', html::specialchars($v));
			}
			printf('</select>');
		} else {
			if ( $dflt ) {
				$default .= '<option value="' . html::specialchars($dflt) . '" selected="selected">' . html::specialchars($dflt) . '</option>';
			}

			$template = '<select class="'.$class_name.'" id="%1$s" name="%1$s" data-type="%2$s" data-filterable="true" %3$s>' .
				$default . '</select>';

			printf($template, html::specialchars($select_name), html::specialchars($tokenvalues), $required ? 'data-required="true"' : '');
		}

		return true;
	}

	function render( $obj, $token, $value ) {
		$sel_tmp = $obj->properties['special'][$token];

		self::make_selection($sel_tmp, $value,
				   $this->get_id($obj, $token),
				   'xlarge', in_array($token, $obj->properties['required']));

		if ($token==='check_command'){
			echo "<input type=\"button\" class=\"button syntax-help20\" id=\"syntax_button\" value=\"Syntax help\" title=\"Show syntax help for selected command\" onclick=\"toggle_command_info(this.form, '', '".$obj->id."', true);\">";
			echo "<input type=\"button\" class=\"button edit-check-command20\" title=\"Edit selected Check Command\" onclick=\"return editCheckCommand(this.form);\" value=\"Edit check command\">";
			echo "<div id=\"commandInfo\"></div>";
		}
		if ($token === 'template') {
			echo '<input type="button" class="button force_template_btn" value="Force template values" title="Force values from template to current object" />';
			echo ' <input type="button" class="show_template_values_btn button" value="View template values" title="View template values for current object" />';
			if(!isset($sel_tmp[$value])) {
				return;
			}
			printf(' (current template: <span id="current_template">%s</span>)', $value);
		}
	}

	static function validate($obj, $token, $value) {
		$spec = $obj->properties['special'];
		$req_vars = $obj->properties['required'];
		// special case for 'not selected' or if object is array
		if (($value === '0' && !in_array($value, $req_vars)) || is_array($spec))
			return $value;
		$class = $spec . '_nacoma_Model';
		if (!$class::object_exists($obj->obj[$token]))
			throw new widget_validation_exception("Supposed to link to {$spec[$token]} type object, but the object linked to doesn't exist");
		return $value;
	}
}
